import { createRoot } from 'react-dom/client'
import { lazy, Suspense } from 'react'
import App from './App.tsx'
import './index.css'

// Funzione per ottimizzare il caricamento delle immagini
const optimizeImageLoading = () => {
  const images = document.querySelectorAll('img');
  images.forEach(img => {
    if (img instanceof HTMLImageElement) {
      if (!img.hasAttribute('loading') && !img.hasAttribute('fetchpriority')) {
        img.loading = 'lazy';
      }
      if (!img.hasAttribute('alt')) {
        img.alt = 'DSR Pulizie Milano - Servizi di pulizia professionale';
      }
      // Aggiungiamo dimensioni esplicite per migliorare CLS (Cumulative Layout Shift)
      if (!img.hasAttribute('width') && !img.hasAttribute('height') && !img.hasAttribute('style')) {
        img.setAttribute('width', '100%');
        img.style.aspectRatio = 'auto';
      }
      // Aggiungiamo il decoding asincrono
      img.decoding = 'async';
    }
  });
};

// Misurazioni delle performance
const measurePerformance = () => {
  if ('performance' in window && 'getEntriesByType' in performance) {
    try {
      // Registriamo le metriche Web Vitals
      new PerformanceObserver((entryList) => {
        const entries = entryList.getEntries();
        entries.forEach(entry => {
          // Invia le metriche alla console per debug
          console.debug('[Performance]', entry.name, entry.startTime, entry.duration);
        });
      }).observe({type: 'largest-contentful-paint', buffered: true});
      
      new PerformanceObserver((entryList) => {
        const entries = entryList.getEntries();
        entries.forEach(entry => {
          // Using type assertion to access layout shift specific properties
          const layoutShiftEntry = entry as LayoutShiftAttribution;
          if (layoutShiftEntry.hadRecentInput) return;
          console.debug('[Performance] CLS:', layoutShiftEntry.value);
        });
      }).observe({type: 'layout-shift', buffered: true});
      
      new PerformanceObserver((entryList) => {
        const entries = entryList.getEntries();
        entries.forEach(entry => {
          // Using type assertion for first-input specific properties
          const firstInputEntry = entry as FirstInputTiming;
          console.debug('[Performance] FID:', firstInputEntry.processingStart - firstInputEntry.startTime);
        });
      }).observe({type: 'first-input', buffered: true});
    } catch (e) {
      console.error('Performance API error:', e);
    }
  }
};

// Define the missing interfaces for Performance Observer entries
interface LayoutShiftAttribution extends PerformanceEntry {
  hadRecentInput: boolean;
  value: number;
}

interface FirstInputTiming extends PerformanceEntry {
  processingStart: number;
}

// Ottimizzazione delle risorse per Core Web Vitals
const optimizeCoreWebVitals = () => {
  // Implementiamo Intersection Observer per caricare risorse solo quando necessario
  if ('IntersectionObserver' in window) {
    const lazyLoadElements = document.querySelectorAll('.lazy-load');
    
    if (lazyLoadElements.length > 0) {
      const observer = new IntersectionObserver((entries) => {
        entries.forEach(entry => {
          if (entry.isIntersecting) {
            const element = entry.target;
            if (element instanceof HTMLImageElement || element instanceof HTMLIFrameElement) {
              const dataSrc = element.getAttribute('data-src');
              if (dataSrc) {
                element.src = dataSrc;
                element.removeAttribute('data-src');
              }
            } else {
              element.classList.add('visible');
            }
            observer.unobserve(element);
          }
        });
      }, { rootMargin: '100px' });
      
      lazyLoadElements.forEach(element => {
        observer.observe(element);
      });
    }
  }
};

// Inizializzazione dell'app con ottimizzazioni avanzate
const initApp = () => {
  // Monitoraggio performance
  measurePerformance();
  
  // Rendering dell'app
  const root = createRoot(document.getElementById("root")!);
  root.render(
    <Suspense fallback={<div className="flex items-center justify-center min-h-screen">Caricamento...</div>}>
      <App />
    </Suspense>
  );
  
  // Ottimizzazione post-render
  document.addEventListener('DOMContentLoaded', () => {
    // Ottimizzazioni di primo caricamento
    optimizeCoreWebVitals();
    
    // Deferred loading di script e risorse non critiche
    setTimeout(() => {
      optimizeImageLoading();
    }, 100);
  });
};

// Avvia l'applicazione
initApp();
